import { format } from 'util';
import ts from 'typescript';
import { appendDotCjs, cloneProgram } from "./append.cjs.js";
import { appendDotCjsDirect } from "./append.cjs.heft.js";
import { appendDotJs } from "./append.js.js";
import { getDebug } from "./debug.js";
import { selfCreatedProgram } from "./preventLoop.js";
let lastProgram;
let cachedTransformer;
function makeLogger(logger, verbose) {
    return {
        debug(msg, ...args) {
            if (verbose) {
                logger.terminal.writeLine(format(msg, ...args));
            }
            else {
                logger.terminal.writeVerboseLine(format(msg, ...args));
            }
        },
        error(msg) {
            logger.terminal.writeWarningLine(msg);
        },
    };
}
export default function typescriptTransformerDualPackage(program, pluginOptions) {
    const verbose = pluginOptions.verbose || process.env.NODE_DEBUG?.includes('tramsform') || false;
    const logger = pluginOptions.logger ? makeLogger(pluginOptions.logger, verbose) : getDebug(verbose);
    if (selfCreatedProgram in program) {
        return () => {
            (node) => node;
        };
    }
    if (program === lastProgram) {
        return cachedTransformer;
    }
    lastProgram = program;
    if (pluginOptions.logger) {
        let transformer;
        const mdl = program.getCompilerOptions().module;
        logger.debug('emit program type ' + ts.ModuleKind[mdl]);
        if (mdl === ts.ModuleKind.CommonJS) {
            if (pluginOptions.cjs === false) {
                logger.debug('config cjs is false');
                transformer = noop;
            }
            else {
                const ext = pluginOptions.cjs || '.cjs';
                logger.debug('creating program for %s', ext);
                transformer = function transformer(transformationContext) {
                    logger.debug('[trans] NewContext');
                    return appendDotCjsDirect(program, transformationContext, logger);
                };
            }
        }
        else if (mdl >= ts.ModuleKind.ES2015 && mdl <= ts.ModuleKind.ESNext) {
            if (pluginOptions.mjs === false) {
                logger.debug('config mjs is false');
                transformer = noop;
            }
            else {
                const ext = pluginOptions.mjs || '.js';
                logger.debug('creating program for %s', ext);
                transformer = function transformer(transformationContext) {
                    logger.debug('[trans] NewContext');
                    return appendDotJs(program, transformationContext, logger);
                };
            }
        }
        else {
            logger.debug('creating noop program');
            transformer = noop;
        }
        return transformer;
    }
    else {
        const shadowCompilerOptions = pluginOptions.compilerOptions || {};
        shadowCompilerOptions.declaration = false;
        shadowCompilerOptions.declarationMap = false;
        // console.error('[dual-package]: ttsc: creating shadow program!');
        const shadowProgram = cloneProgram(program, shadowCompilerOptions, logger);
        const appendDotCjsCall = appendDotCjs(shadowProgram, logger);
        return (cachedTransformer = function transformer(transformationContext) {
            logger.debug('[trans] NewContext');
            const appendDotJsCall = appendDotJs(program, transformationContext, logger);
            return (sourceFile) => {
                appendDotCjsCall(sourceFile);
                return appendDotJsCall(sourceFile);
            };
        });
    }
}
function noop() {
    return (s) => s;
}
//# sourceMappingURL=register.js.map